@@ -1,8 +1,10 @@
 package Sub::Uplevel;
 
+use 5.006;
 use strict;
-use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '0.19';
+our $VERSION = '0.20';
+$VERSION = eval $VERSION;
+
 
 # We must override *CORE::GLOBAL::caller if it hasn't already been 
 # overridden or else Perl won't see our local override later.
@@ -12,8 +14,8 @@ if ( not defined *CORE::GLOBAL::caller{CODE} ) {
 }
 
 require Exporter;
-@ISA = qw(Exporter);
-@EXPORT = qw(uplevel);
+@Sub::Uplevel::ISA = qw(Exporter);
+@Sub::Uplevel::EXPORT = qw(uplevel);
 
 =head1 NAME
 
@@ -79,18 +81,27 @@ you can do this:
         return @out;
     }
 
+C<uplevel> will issue a warning if C<$num_frames> is more than the current call
+stack depth.
 
 =cut
 
-use vars qw/@Up_Frames $Caller_Proxy/;
 # @Up_Frames -- uplevel stack
 # $Caller_Proxy -- whatever caller() override was in effect before uplevel
+our (@Up_Frames, $Caller_Proxy);
+
+sub _apparent_stack_height {
+    my $height = 1; # start above this function 
+    while ( 1 ) {
+        last if ! defined scalar $Caller_Proxy->($height);
+        $height++;
+    }
+    return $height - 1; # subtract 1 for this function
+}
 
 sub uplevel {
     my($num_frames, $func, @args) = @_;
     
-    local @Up_Frames = ($num_frames, @Up_Frames );
-    
     # backwards compatible version of "no warnings 'redefine'"
     my $old_W = $^W;
     $^W = 0;
@@ -103,6 +114,13 @@ sub uplevel {
     # restore old warnings state
     $^W = $old_W;
 
+    if ( $num_frames >= _apparent_stack_height() ) {
+      require Carp;
+      Carp::carp("uplevel $num_frames is more than the caller stack");
+    }
+
+    local @Up_Frames = ($num_frames, @Up_Frames );
+    
     return $func->(@args);
 }
 
@@ -297,5 +315,4 @@ Tcl's uplevel() at http://www.scriptics.com/man/tcl8.4/TclCmd/uplevel.htm
 
 =cut
 
-
 1;
